/** Ground defense game.  Save the Earth from being demolished by falling rocks!! <br>
 *  Created by the Bryn Mawr College CS 110-01 Class on Feb 24, 2011
 */


/////////////// MAIN CONTROLLER AND PROCESSING FUNCTIONS ///////////////
 
// stores the height of the ground
int groundHeight = 40;

// the ground cannon
Ship myShip;

// an array of all the fired missles
Missle[] myMissles = new Missle[0];

Rock[] myRocks = new Rock[0];

float rateOfNewRocks = 0.01;
 
/** Processing setup function
 * Initialize the canvas and create the ship
 */
void setup() {
  size(500,500);
  smooth();
  myShip = new Ship(height, color(255,0,0)); 
}

/** Processing draw function */
void draw() {
  // clear screen, then draw sky and ground
  background(0);
  fill(0,255,0);
  rect(0, height-groundHeight,  width, groundHeight);
  
  // display the ship
  myShip.display();
  
  // display the missles
  for (int i=0; i<myMissles.length; i++) {
    if (myMissles[i] != null) {
      myMissles[i].display();
      myMissles[i].step();
      // test whehter the missle has vanished from sight.  If so, delete it
      boolean deleteMissle = myMissles[i].vanish();
      if (deleteMissle) {
        myMissles[i] = null;
      }
    }
  }
  
  // display the rocks
  for (int i=0; i < myRocks.length; i++) {
    if (myRocks[i] != null) {
      myRocks[i].display();
      myRocks[i].step();
    }
  }
  
  if (random(1) < rateOfNewRocks) {
    generateRock();
  }
  
  
  // test for impacts and remove impacting objects
  for (int r=0; r < myRocks.length; r++) {
    for (int m=0; m < myMissles.length; m++) {
      if (myRocks[r] != null && myMissles[m] != null) {
        boolean impact = myRocks[r].impact(myMissles[m]);
        if (impact) {
          myMissles[m] = null;
          myRocks[r] = null;
        }
      }
    }
  }
  
  // test for end game
  for (int r=0; r < myRocks.length; r++) {
    if (myRocks[r] != null) {
      if (myRocks[r].y >= height-groundHeight) {
        println("GAME OVER!!!");
        rateOfNewRocks = 1;
      }
    }
  }
}


/** The processing mousePressed function
 *  Whenever the mouse is pressed, the ship should fire a missle.
 */
void mousePressed() {
  Missle m = myShip.fire();
  myMissles = (Missle[]) append(myMissles, m);
}


void generateRock() {
  Rock r = new Rock(random(width), 20+random(20), color(100+random(155), 100+random(155), 100+random(155)));
  myRocks = (Rock[]) append(myRocks, r);
}


/////////////// SHIP CLASS ///////////////

/** The ground ship that can fire missles
 */
class Ship {
  
  // the x-coordinate location of the ship
  int x;
  
  // the y-coordinate location of the ship
  int y;
  
  // the width of the ship
  int base = 40;
  
  // the height of the ship
  int height = 80;
  
  // the color for the ship
  color c;
  
  /** Constructor
   * @param initialY - the initial y-locaiton for the bottom of the ship
   * @param shipColor - the color for the ship
   */
  Ship (int initialY, color shipColor) {
    y = initialY;
    c = shipColor;
  }
  
  /** Display the ship */
  void display() {
    // get the mouse position
    x = mouseX;
    // display the object at that location
    fill(c);
    beginShape();
    vertex(x, y-height);
    vertex(x + base/2, y);
    vertex(x - base/2, y);
    endShape(CLOSE);
  }
  
  /** Fire a missle
   * @return the fired missle
   */
  Missle fire() {
    return new Missle(x,y-height);
  }
}



////////////////////// MISSLE CLASS ////////////////////////

/** Represents a missle that goes upwards at a constant speed.
 */
class Missle {
   int x;
   int y;
   int size = 5;

   /** Constructor
    * @param initialX - the initial X coordinate of the missle
    * @param initialY - the initial Y coordinate of the missle
    */   
   Missle(int initialX, int initialY) {
     x = initialX;
     y = initialY;
   }
   
   /** Display the missle */
   void display() {
     fill(255);
     ellipse(x,y,size,size);
   }
   
   /** Move the missle upwards by one step */
   void step() {
     y-=2;
   }
 
   /** Determine whether the missle should continue to be drawn.
    * @return true if the missle has vanished from sight, false otherwise
    */
   boolean vanish() {
     return (y+size/2 <= 0);
   }  
}


class Rock {

  float x, y, w;
  float m;
  color c;
  float speed;
  float gravity;
  float delta;
  
  Rock(float xPosition, float tw, color tc) {
    x = xPosition;
    y = 0;
    w = tw;
    m = 0;
    delta = random(0.8, 0.95);
    c = tc;
    speed = random(0.5, 1.0);
    gravity = 0.01;
  }
  
  void step() {
    x = x + m;
    y = y + speed;
    speed = speed + gravity;
  }

  void display() {
    fill(c);
    noStroke();
    ellipse(x, y, w, w);
  }
  
  float distance(float x1, float y1, float x2, float y2) {
    float deltaX =  x1 - x2;
    float deltaY =  y1 - y2;
    return sqrt(deltaX * deltaX + deltaY * deltaY);
  }
  
  boolean impact(Missle m) {
    return (distance(x,y,m.x,m.y) <= w/2);
  }
} 
  
  
  
  
  
  
  

