/**
Bryn Mawr College, Department of Computer Science <br>
Game developed in class, Fall 2012, section 001 <br>
Move with arrow keys and fire with space bar. <br>
Must click in screen first before keys will function.
*/

Asteroid[] obstacles;
int numAsteroids = 10;
Ship ship;
Laser laser;

void setup() {
  size(600,600);
  smooth();
  obstacles = new Asteroid[numAsteroids];
  for (int i=0; i<obstacles.length; i++) {
    obstacles[i] = new Asteroid();
  }
  
  ship = new Ship(width/2, height/2);
}

void draw() {
  background(0);
  
  // display the asteroids
  for (int i=0; i<obstacles.length; i++) {
    obstacles[i].display();
    obstacles[i].stepAnimation();
    if (obstacles[i].collision(ship)) {
      background(255,0,0);
    }
  }
  
  // display the laser
  if (laser != null) {
    laser.display();
    laser.stepAnimation();
  }
  
  // display the ship
  ship.display();
  ship.stepAnimation();
}

void keyPressed() {
  switch(keyCode) {
    case UP:
      ship.moveUp();
      break;
    case DOWN:
      ship.moveDown();
      break;
    case LEFT:
      ship.moveLeft();
      break;
    case RIGHT:
      ship.moveRight();
      break;
  }
  if (key == ' ') {
    laser = ship.fire();
  }
}



class Asteroid {
  float x, y, xSpeed, ySpeed;
  int maxSpeed = 2;
  int size;
  
  Asteroid() {
    initialize();
  }
  
  void initialize() {
    // pick the size
    size = (int) random(5,50);
    
    x = random(0, width);
    y = random(0, height);
    xSpeed = random(-maxSpeed, maxSpeed);
    ySpeed = random(-maxSpeed, maxSpeed);
    
    // generate the asteroid along the edge
    int edge = (int) random(0,4);
    switch(edge) {
      // left edge
      case 0:
        x = -size/2;
        xSpeed = abs(xSpeed); // make xSpeed positive
        break;
      // top edge
      case 1:
        y = -size/2;
        ySpeed = abs(ySpeed); // make positive
        break;
      // right edge
      case 2:
        x = width + size/2;
        xSpeed = -abs(xSpeed); // make negative
        break;
      // bottom edge
      case 3:
        y = height + size/2;
        ySpeed = -abs(ySpeed); // make negative
    }
    

    
  }
  
  void display() {
    fill(255);
    ellipse(x,y,size,size);
  }
  
  void stepAnimation() {
    x += xSpeed;
    y += ySpeed;
    
    // check for asteroid being off screen
    if (x > width+size/2 || x < -size/2 || y > height+size/2 || y < -size/2) {
      initialize();
    }
  }
  
  boolean collision(Ship s) {
    float distance = dist(x,y, s.x,s.y);
    if (distance-10 <= size/2) {
      return true;
    } else {
      return false;
    }
  
  }
}
class Laser {
  
  float x, y, speed;
  
  Laser() {
    x = ship.x;
    y = ship.y;
    speed = 1;
  }
  
  void display() {
    stroke(255,0,0);
    strokeWeight(3);
    line(x,y,x,y+10);
  }
  
  void stepAnimation() {
    y -= speed;
  }
}
class Ship {
  float x, y, vx, vy;
  float speedChangeRate = 0.2;

  Ship(float tempX, float tempY) {
    x = tempX;
    y = tempY;
    vx = 0;
    vy = 0;
  }
  
  void display() {
    fill(0,0,255);
    ellipse(x,y,20,60);
    ellipse(x-10,y+30,10,30);
    ellipse(x+10,y+30,10,30);
  }
  
  void stepAnimation() {
    x += vx;
    y += vy;
  }
  
  void moveUp() {
    vy -= speedChangeRate;
  }
  
  void moveDown() {
    vy += speedChangeRate;
  }
  
  void moveLeft() {
    vx -= speedChangeRate;
  }
  
  void moveRight() {
    vx += speedChangeRate;
  }
  
  Laser fire() {
    return new Laser();
  }
  
}


