/**
Dianna Xu <br>
Bryn Mawr College, Department of Computer Science <br>
When two circles collide, an emergent circle is drawn. <br>
Original generating circles not displayed. <br>
Inspired by ideas from Generative Art by Matt Pearson
*/

Circle[] cs = new Circle[200];

void setup() {
  size(800, 800);
  smooth();
  background(255);

  for(int i=0; i<cs.length; i++) {
    if (i%10 == 0) {
      cs[i] = new Circle(random(width), random(height), random(150, 200), i);
    }
    else {
      cs[i] = new Circle(random(width), random(height), random(50), i);
    }
  }
} // end setup()

void draw() {
  background(255);
  for(int i=0; i<cs.length; i++) {
    //cs[i].display();
    cs[i].update(cs);
  }
} // end draw()

class Circle {
  PVector center;
  float radius;
  PVector v;
  color c;
  int id;

  Circle (float x, float y, float radius, int id) {
    center = new PVector(x, y, 0);
    this.radius = radius;
    v = new PVector(random(-1, 1), random(-1, 1));
    c = color(random(255));
    this.id = id;
  }

  void display() {
    fill(c);
    stroke(c);
    ellipse(center.x, center.y, radius*2, radius*2);
  }

  void update(Circle[] cs) {
    center.add(v);

    for (int i=id+1; i<cs.length; i++) {
      float dist = PVector.dist(center, cs[i].getCenter());
      float tangentDist = radius + cs[i].getRadius();

      if (dist <= tangentDist) {
        float overlap = tangentDist-dist;
        float strokeA = map(overlap, 0, tangentDist, 0, 255);          
        float strokeW = map(overlap, 0, tangentDist, 0, 5);
        //strokeWeight(0.01*strokeW);
        strokeWeight(strokeW);
        //noStroke();
        //fill((c + cs[i].c)/2, strokeA*0.5);
        noFill();
        stroke((c + cs[i].getColor())/2, strokeA);   
        PVector newCenter = PVector.add(center, cs[i].getCenter());
        newCenter.div(2);
        ellipse(newCenter.x, newCenter.y, overlap, overlap);
        //rect(newCenter.x, newCenter.y, overlap, overlap);
      }
    }

    if (center.x < 0 || center.x > width) {
      v.x = -v.x;
    }
    if (center.y < 0 || center.y > height) {
      v.y = -v.y;
    }
  } // end update()
  
  PVector getCenter() {return center;}
  float getRadius() {return radius;}
  color getColor() {return c;}
} // end class Circle


