/**
Bryn Mawr College, Department of Computer Science <br>
Inspired by http://weblog.jamisbuck.org/2011/1/12/maze-generation-recursive-division-algorithm
*/

int N = 50;     // Grid dimension
int gsize = 10; // Grid size

int V = 1;      // Vertical constant
int H = 2;      // Horizontal constant

ArrayList regions = new ArrayList();

class Region {
  int r1, c1, r2, c2;
  Region(int _r1, int _c1, int _r2, int _c2) {
    r1 = _r1;
    c1 = _c1;
    r2 = _r2;
    c2 = _c2;
  }
}

void setup() {
  // Setup sketch
  //size(N*gsize+1, N*gsize+1);
  size(501, 501);
  //frameRate(10);
  
  background(255);
  stroke(0);
  
  // Push top region on list
  regions.add(0, new Region(0,0,N,N));
  
  //noLoop();
}

void draw() {
  // Each call to draw pops a new region off the list and subdivides it.
  if (regions.size() > 0) {
    Region g = (Region)regions.remove(0);
    divide(g.r1, g.c1, g.r2, g.c2);
  }
}

void mousePressed() {
  redraw();
}

// Divide the region given upper left and lower right 
// grid corner points
void divide(int r1, int c1, int r2, int c2) {
  int cr, rr;

  // Get divide direction (V, H or 0)
  int dir = divDir(r1, c1, r2, c2);

  // Divide in vertical direction  
  if (dir == V) {
    
    // Calculate wall and opening locations
    cr = randomInt(c1+1, c2-1);
    rr = randomInt(r1, r2-1);
    
    // Draw wall  
    gridLine(cr,r1,cr,rr);
    gridLine(cr,rr+1,cr,r2);
    
    // Recursively divide two subregions
    regions.add(0, new Region(r1,cr,r2,c2));
    regions.add(0, new Region(r1,c1,r2,cr));
    
//    divide(r1,c1,r2,cr);
//    divide(r1,cr,r2,c2);

  // Divide in horizontal direction
  } else if (dir == H) {
    
    // Calculate wall and opening locations
    cr = randomInt(c1, c2-1);
    rr = randomInt(r1+1, r2-1);
    
    // Draw wall  
    gridLine(c1,rr,cr,rr);
    gridLine(cr+1,rr,c2,rr);
    
    // Recursively divide two subregions
    // Recursively divide two subregions
    regions.add(0, new Region(rr,c1,r2,c2));
    regions.add(0, new Region(r1,c1,rr,c2));
    
//    divide(r1,c1,rr,c2);
//    divide(rr,c1,r2,c2);
      
  // No division. We're done.
  } else {
    return;
  }
}

// Determine the direction for dividing the region
// based on which dimension is smaller.
// Stop when too small.
int divDir(int r1, int c1, int r2, int c2) {
  int dr = r2 - r1;        // Calculate deltas
  int dc = c2 - c1;
  if (dr <= 1 || dc <= 1)  // Too small
    return 0;              // No division
  else if (dr < dc)        // Flat and wide
    return V;              // Vertical division
  else                     // Tall and narrow
    return H;              // Horizontal division
}

// Return a random integer in the range [min, max]
int randomInt(int min, int max) {
  return round(random(min-0.5,max+0.5));
}

// Draw a line on a grid segment given grid points
void gridLine(int r1, int c1, int r2, int c2) {
  line(r1*gsize, c1*gsize, r2*gsize, c2*gsize);
}

