/**
   Dianna Xu <br>
   Bryn Mawr College, Department of Computer Science <br>
   Adapted from code by Mark Russo <br>
   Magnification at mouse point that demos pixelization. Mouse scroll increase/decreases zoom. Arrow keys modify circle diameter and zoom levels. <br>
   Must click in screen first before keys will function. 
*/

/* @pjs preload="tree.jpg"; */
/* @pjs globalKeyEvents="true"; */

PImage img;
CMagnify glass;
float zoom = 2.0;
int diameter = 150;

void setup() {
  size(600, 400);

  // http://www.freefoto.com/images/15/19/15_19_8---Tree--Northumberland_web.jpg
  img = loadImage("tree.jpg");
  smooth();

  // Create a new magnifying glass
  glass = new CMagnify(this);
}

void draw() {
  // Draw something
  image(img, 0, 0);
  //background(255);
  //line(0,0,400,300);
  //ellipse(150, 150, 100, 50);

  // Draw the magnifying glass at the mouse position.
  glass.draw(mouseX, mouseY);
}

// Magnify on mouse wheel
void mouseWheel(MouseEvent evt) {
  float delta = evt.getAmount();
  zoom -= delta;
  if (zoom < 1.0) zoom = 1.0;
}

// Javascript version
void mouseScrolled() {
  if (mouseScroll != 0)
    zoom += mouseScroll;
  if (zoom < 1.0) zoom = 1.0;
}

// Increase zoom on up-arrow. Decrease zoom on down-arrow.
// Increase diameter on right-arrow. Decrease diameter on left arrow.
// Must click in screeen for keys to work in Javascript, focus is messed up
void keyPressed() {
  if (keyCode == UP) {
    zoom += 1.0;
  }
  else if (keyCode == DOWN) {
    zoom -= 1.0;
    if (zoom < 1.0) zoom = 1.0;
  }
  else if (keyCode == RIGHT) {
    diameter += 4;
  }
  else if (keyCode == LEFT) {
    diameter -= 4;
    if (diameter < 50) diameter = 50;
  }
}

// Magnifying glass class
class CMagnify {
  private PApplet apl = null;
  // Constructor
  CMagnify(PApplet apl) {
    this.apl = apl;    
  }
  

  // Draw the magnifying glass at the given location, size and magnification.
  public void draw(int x, int y) {
    int px, pz;
    int radius = diameter/2;
  
    // Load current image into pixel array
    PImage img = apl.get();
    img.loadPixels();
    int maxpz = img.pixels.length-1;
    
    // Create image for zoom
    PImage g = new PImage(diameter, diameter);
    g.loadPixels();
    
    // Build zoomed image
    for (int j=0; j<diameter; j++) {
      for (int i=0; i<diameter; i++) {
        try {
          px = j*diameter + i;
          float fx = (float)(i-radius);
          float fy = (float)(j-radius);
          float dst = dist(fx, fy, 0.0, 0.0);
          if (dst <= radius)      // Zoom if within radius
          {
            int zx = x + round(fx/zoom);
            int zy = y + round(fy/zoom);
            pz = zy*img.width + zx;
            if (pz >= 0 && pz <= maxpz) g.pixels[px] = img.pixels[pz];
          }
          else                    // Copy pixels if outside radius
          {
            int zx = x + i-radius;
            int zy = y + j-radius;
            pz = zy*img.width + zx;
            if (pz >= 0 && pz <= maxpz) g.pixels[px] = img.pixels[pz];
          }
        }
        catch(ArrayIndexOutOfBoundsException e) {
          // Ignore array out of bounds exceptions
        }
        catch(Exception e) {
          // Ignore exceptions
        }
      }
    }
  
    // Update pixels and draw
    g.updatePixels();
    apl.image(g, x-radius, y-radius);
    
    // Add outline
    apl.noFill();
    apl.stroke(0);
    apl.ellipse(x, y, diameter, diameter);
  }
}

