/**
Bryn Mawr College, Department of Computer Science <br>
*/


CSlider slider;
CButton upButton;
CButton downButton;

void setup() {
  size(700,400);
  slider = new CSlider(50, 50, 25, 0, "");
  upButton = new CButton( 10, 50, 30, 30, "^");
  downButton = new CButton( 10, 80, 30, 30, "v");
}

void draw() {
  background(232);
  
  textSize(32);
  
  // Draw binary number
  textAlign(RIGHT);
  text("Binary:", 240, 75);
  text("Decimal:", 240, 115);
  text("Hex:", 240, 155);
  
  fill(0);
  textAlign(LEFT);
  String tmp = binary(slider.val);
  if (tmp.length() < 8)
    tmp = pad("0", 8-tmp.length()) + tmp;
  else if (tmp.length() > 8)
    tmp = tmp.substring(tmp.length()-8);  // Req'd for p.js
  text(tmp, 250, 75);
  text(str(slider.val), 250, 115);
  tmp = hex(slider.val);
  tmp = tmp.substring(tmp.length()-2);    // Req'd for p.js
  text(tmp, 250, 155);
  
  // Draw controls
  slider.draw();
  upButton.draw();
  downButton.draw();
  
  // Update state
  if (upButton.clicked() == true) { slider.setVal( slider.val + 1 ); }
  if (downButton.clicked() == true) { slider.setVal( slider.val - 1 ); }
}

void mousePressed() {
  slider.mousePressed();
  upButton.mousePressed();
  downButton.mousePressed();
}

void mouseDragged() {
  slider.mouseDragged();
}

void mouseReleased() {
  slider.mouseReleased();
  upButton.mouseReleased();
  downButton.mouseReleased();
}

void mouseClicked() {
  upButton.mouseClicked();
  downButton.mouseClicked(); 
}

String pad(String ch, int num) {
  String chars = "";
  for (int i=0; i<num; i++) chars += ch;
  return chars;
}

// - - - - - - - - - - - 
// CButton object implements a simple button
class CButton {
  int x, y;
  float width = 100.0;
  float height = 30.0;
  String label;
  color   fillColor = color(247);
  color   strokeColor = color(0);
  color   textColor = color(0);
  int     fontSize = 20;
  boolean _clicked = false;
  
  // Button state
  boolean pressed = false;

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  CButton(int x, int y, float width, float height, String label) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.label = label;
  }
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
  void draw() {
    // Draw base rectangle
    stroke(strokeColor);
    strokeWeight(1);
    fill(fillColor);
    rect(x, y, width, height);
    
    // Compute highlights and y offset
    int highlight1 = 255;
    int highlight2 = 192;
    int yoffset = 0;
    if (pressed) {
      highlight1 = 192;
      highlight2 = 255;
      yoffset = 1;
    }
    
    // Draw highlights
    stroke(highlight1);
    rect(x+2, y+1, width-4, 1);
    stroke(highlight2);
    rect(x+2, y+height-2, width-4, 1);
    
    // Draw label
    textAlign(CENTER);
    textSize(fontSize);
    fill(textColor);
    text(label, x, y+yoffset, width, height);
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  void mousePressed()  {
    if (containsPoint(mouseX, mouseY) == true) { pressed = true; }
  }
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  void mouseReleased() {
    pressed = false;
  }

  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  void mouseClicked() {
    if (containsPoint(mouseX, mouseY) == true) { _clicked = true; }
  }
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  boolean clicked() {
    // Returns the clicked state of the button and then resets
    boolean tmp = _clicked;
    _clicked = false;
    return tmp;  
  }
  
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  boolean containsPoint(int x, int y) {
    // Return true if CSlider contains given point.
    if (x < this.x) return false;
    if (y < this.y) return false;
    if (x > this.x + this.width) return false;
    if (y > this.y + this.height) return false;
    return true;
  }
}

// - - - - - - - - - - - 
// CSlider object implements a simple slider that allows user to select a value in range [0..255]
class CSlider {
  int x, y;
  int width;
  int height = 256;
  int val;
  String label;
  boolean captured = false;
 
  CSlider(int x, int y, int width, int val, String label) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.val = val;
    this.label = label;
  }
  
  void setVal(int v) {
    // Set slider value, clipping to [0..255]
    if (v < 0) v = 0;
    if (v > 255) v = 255;
    val = v;
  }
  
  void draw() {
    // Draw labels
    fill(255,255,255,127);
    noStroke();
    rect(x, y, width+1, 35);
    rect(x, y+height, width+1, 25);
    fill(0);
    textAlign(CENTER);
    textSize(24);
    text(this.label, x, y, width, -35);
    textSize(16);
    text(str(val), x, y+height+5, width, 25);
    
    // Draw slider
    fill(255);
    stroke(127);
    strokeWeight(2);
    rect(x, y, width, height);
    fill(127,0,0);
    rect(x-3, y+height-val-3, width+5, 7);
  }
  
  void mousePressed() {
    // If mouse pressed in CSlider, set new value.
    if (!containsPoint(mouseX, mouseY)) return;
    captured = true;
    setVal( y+height-mouseY );
  }
  
  void mouseDragged() {
    // If dragging and this CSlider has captured mouse, 
    // set new value.
    if (!captured) return;
    setVal( y+height-mouseY );
  }
  
  void mouseReleased() {
    // Always release capture.
    captured = false;
  }
  
  boolean containsPoint(int x, int y) {
    // Return true if CSlider contains given point.
    if (x < this.x) return false;
    if (y < this.y) return false;
    if (x > this.x + this.width) return false;
    if (y > this.y + this.height) return false;
    return true;
  }
}


