/**
   Dianna Xu <br>
   Adapted from code by Mark F. Russo <br>
   Bryn Mawr College, Department of Computer Science <br>
   Shows RGBA values and how they affect color
*/

CSlider slideR;
CSlider slideG;
CSlider slideB;
CSlider slideA;

PImage img;

// - - - - - - - - - - - 
void setup() {
  size(600, 400);
  smooth();
  
  // Create one CSlider object for R, G, B and A color components
  slideR = new CSlider(90, 80, 30, 0, "R");
  slideG = new CSlider(130, 80, 30, 0, "G");
  slideB = new CSlider(170, 80, 30, 0, "B");
  slideA = new CSlider(210, 80, 30, 255, "A");
  
  // Load image for background
  img = loadImage("tree.jpg");
}

void draw() {  
  // Draw background.
  // Note: Anything can be drawn here to investigate how the alpha channel works,
  //       including a simple background fill.
  //background(255,0,0);
  image(img, 0, 0);
  
  slideR.draw();      // Draw sliders
  slideG.draw();
  slideB.draw();
  slideA.draw();
  
  stroke(0);          // Draw color box
  strokeWeight(1);
  fill(slideR.val, slideG.val, slideB.val, slideA.val);
  rect(260, 80, 256, 256);
}

void mousePressed() {
  // Pass all mousePressed events to CSliders
  slideR.mousePressed();
  slideG.mousePressed();
  slideB.mousePressed();
  slideA.mousePressed();
}

void mouseDragged() {
  // Pass all mouseDragged events to CSliders
  slideR.mouseDragged();
  slideG.mouseDragged();
  slideB.mouseDragged();
  slideA.mouseDragged();
}

void mouseReleased() {
  // Pass all mouseReleased events to CSliders
  slideR.mouseReleased();
  slideG.mouseReleased();
  slideB.mouseReleased();
  slideA.mouseReleased();
}

// - - - - - - - - - - - 
// CSlider object implements a simple slider that allows user to select a value in range [0..255]
class CSlider {
  int x, y;
  int width;
  int height = 256;
  int val;
  String label;
  boolean captured = false;
 
  CSlider(int x, int y, int width, int val, String label) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.val = val;
    this.label = label;
  }
  
  void setVal(int v) {
    // Set slider value, clipping to [0..255]
    if (v < 0) v = 0;
    if (v > 255) v = 255;
    val = v;
  }
  
  void draw() {
    // Draw labels
    fill(255,255,255,127);
    noStroke();
    rect(x, y, width+1, -35);
    rect(x, y+height, width+1, 25);
    fill(0);
    textAlign(CENTER);
    textSize(24);
    text(this.label, x, y-25, width, 35);
    textSize(16);
    //text(str(val), x, y+height, width, 25);
    text(str(val), x, y+height+8, width, 25);
    
    // Draw slider
    fill(255);
    stroke(127);
    strokeWeight(2);
    rect(x, y, width, height);
    fill(127,0,0);
    rect(x-3, y+height-val-3, width+5, 7);
  }
  
  void mousePressed() {
    // If mouse pressed in CSlider, set new value.
    if (!containsPoint(mouseX, mouseY)) return;
    captured = true;
    setVal( y+height-mouseY );
  }
  
  void mouseDragged() {
    // If dragging and this CSlider has captured mouse, 
    // set new value.
    if (!captured) return;
    setVal( y+height-mouseY );
  }
  
  void mouseReleased() {
    // Always release capture.
    captured = false;
  }
  
  boolean containsPoint(int x, int y) {
    // Return true if CSlider contains given point.
    if (x < this.x) return false;
    if (y < this.y) return false;
    if (x > this.x + this.width) return false;
    if (y > this.y + this.height) return false;
    return true;
  }
}

