/**
Bryn Mawr College, Department of Computer Science <br>
Word frequence visualization: George Washington State of Union Address, 1790
*/

String inFileName = "GWSoU1-8-1790.txt", raw;
String delimiters = " ,./?;:'\"[{]}\\|=+-_)(*&^%$#@!~";
String[] fileText, tokens, words;
int[] count;
ArrayList stopwords;

PFont f;
float minFS, maxFS;
float x, y;
int maxCount;
float textFS;
float c;

void setup() {
  size(800, 500);
  f = createFont("Broadway", 78, true);
  minFS = 12;
  maxFS = 78;
  textAlign(CENTER);  
  smooth();
  // noLoop();
  frameRate(1);
  // Read the text file, and clean it up into tokens
  // Also read in all stop words
  acquire();
  // mine for word freqencies
  mine();
  maxCount = max(count);
  display();
} 



void draw() {
  display();
} 

void display() {
  background(255);  
  fill(0); 

  for (int i = 0; i < 50; i++) {
    // draw the ith word
    c = map(count[i], 1, maxCount, 127, 50);
    fill(c);
    textFS = map(count[i], 1, maxCount, minFS, maxFS); 
    textFont(f, textFS);
    x = random(50, width-50);
    y = random(35, height-35);
    text(words[i], x, y);
  }
}

void acquire() {
  // Read the text file, first
  fileText = loadStrings(inFileName);
  raw = join(fileText, " ");
  raw = raw.toLowerCase();
  tokens = splitTokens(raw, delimiters);
  println("There are " + tokens.length + " words to process.");

  // Also acquire stop words
  fileText = loadStrings("stopwords.txt");
  stopwords = new ArrayList(fileText.length);

  for (int i=0; i < fileText.length; i++) {
    stopwords.add(fileText[i].toLowerCase());
  }
  println("Read "+stopwords.size()+" stop words.");
}

void mine() {
  //words = new String[tokens.length];
  //count = new int[tokens.length];
  words = new String[0];
  count = new int[0];

  for (int i=0; i < tokens.length; i++) {
    // count the ith token, if it isn't a stop word
    if (!stopwords.contains(tokens[i])) {
      int idx = contains(tokens[i], words);

      if (idx < 0) {
        // a new word, create an entry for it with count 1
        words = append(words, tokens[i]);
        count = append(count, 1);
      }
      else {
        // update the count for ix-th word
        count[idx]++;
      }
    }
  }
  wordCountSort(words, count);

  println("There are "+ words.length +" unique words.");
} // mine

int contains(String t, String[] ws) {
  // Retun the index of t in ws[] or -1
  for (int i=0; i<ws.length; i++) {
    if (t.equals(ws[i]))
      return i;
  }
  return -1;
}

void wordCountSort(String[] ws, int[] cs) {
  int maxIdx;
  int tmp;
  String tmpStr;

  for (int i = 0; i < cs.length-1; i++) {
    maxIdx = i;
    for (int j = i; j<cs.length; j++) {
      if (cs[j] > cs[maxIdx])
        maxIdx = j;
    } 
    // cs[maxIdx] is the current largest count 
    // swap cs[i] and cs[maxIndx]
    tmp  = cs[i];
    cs[i] = cs[maxIdx];
    cs[maxIdx] = tmp;

    // swap the words as well
    tmpStr = ws[i];
    ws[i] = ws[maxIdx];
    ws[maxIdx] = tmpStr;
  }
}


